//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.catalog.commands.*;

import java.util.*;
import java.sql.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import javax.servlet.http.HttpUtils.*;
import javax.servlet.http.*;

import com.ibm.commerce.datatype.*;
import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.catalog.objects.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;

/**
 * This command first ensures that the supplied input <I>PackageId</I>is indeed a reference to 
 * a Package.  After doing so, it determines the distinct PackageAttributes that are needed to resolve the unresolved Package 
 * into an orderable Package.  For each of these PackageAttributes, the command ensures that an appropriate PackageAttributeValue has 
 * been supplied in the <I>AttributeNVPs</I> hashtable.  Once these PackageAttribute name-value pairs have been identified, the command 
 * resolves this information 
 * into a <I>PackageItemId</I> that represents the orderable Package.  If PackageAttributes are missing or are incorrect, the 
 * <I>ErrorMissingAttributeErrorCmd</I> or  <I>ErrorProductAttributeErrorCmd</I> error tasks are launched respectively.
 */
public class PackageResolveSkuCmdImpl  extends TaskCommandImpl implements PackageResolveSkuCmd
{
	/**
	 * The internal copyright field.
	 */
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	private Hashtable ihshAttributeNVPs = null;
	private Long inPackageItemNumber = null;
	private Long inUnresolvedPackageNumber = null;

/**
 * Returns the hashtable that represents the PackageAttribute names and values that will be used to perform the Package resolution. 
 * @return java.util.Hashtable
 */
protected Hashtable getAttributeNVPs() 
{
	return ihshAttributeNVPs;
}
private Enumeration getDistinctAttributes(PackageAccessBean aabPackage, Integer langId) throws com.ibm.commerce.exception.ECException 
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "getDistinctAttributes");
	
	PackageAttributeAccessBean arAttributeList[];
	Vector vecTemp = new Vector();
	
	try
	{
		arAttributeList = aabPackage.getPackageAttributes(langId);

		/*  This needs to be commented out so that resolved packages don't fail
		
		if (arAttributeList.length == 0) // Package has no attributes
		{
			// There aren't any distinct attributes, this is an error situation 
			ECApplicationException exc =  new ECApplicationException(ECMessage._ERR_GETTING_SKU,
											  this.getClass().getName(),"getDistinctAttributes",
											  ErrorMissingAttributeCmd.ERRTASK_NAME);
			
			throw exc;								
		}
		*/
		
		for (int i=0; i < arAttributeList.length; i++)
		{
			String resolvable = arAttributeList[i].getAttribute().getUsage(); 
			if ( resolvable == null || resolvable.equals("1"))
			{
				//we know that this attribute is used for sku resolution, so add it to the list 
				vecTemp.addElement(arAttributeList[i]);
			}
		}
		
	} catch (Exception e) {
		    throw new ECSystemException(ECMessage._ERR_GENERIC,this.getClass().getName(),"getDistinctAttributes",ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}

	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "getDistinctAttributes");

	return vecTemp.elements();
}
/**
 * Returns the PackageId that represents the primary key of the Package that is to be resolved,
 * @return java.lang.Long
 */
protected Long getPackageId ()
{
	return inUnresolvedPackageNumber;
}
/**
 * Returns the PackageId that represents the primary key of the resolved Package
 * @return java.lang.Long
 */
public Long getPackageItemId()
{
	return inPackageItemNumber;
}
public boolean isReadyToCallExecute()
{
	return true;
}
public void performExecute() throws ECException
{

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute");
	super.performExecute();
		
	try
	{
		//Frist we need to make sure that this is a Package
		CatalogEntryAccessBean abCatEntry = new CatalogEntryAccessBean();
		abCatEntry.setInitKey_catalogEntryReferenceNumber(getPackageId().toString());
		if (!abCatEntry.getType().equals(ECConstants.EC_PACKAGE_BEAN))
			return;
			
		PackageAccessBean abPackage = new PackageAccessBean();
		abPackage.setInitKey_catalogEntryReferenceNumber(getPackageId().toString());

		// Let's first get the list of Distinct Attributes for the package
		Enumeration enAttributeList = getDistinctAttributes(abPackage, commandContext.getLanguageId());

		//If the enumeration is empty than this is already a resolved package
		if (enAttributeList.hasMoreElements() == false)
		{
			setPackageItemId(getPackageId());
			return;
		}

		//enAtributeList now contains a collection of distict attributes for the Package
		//we need to:
		//1. Determine if all distinct attributes have been specified by the caller and
		//2. If so, for each attribute, store its associated value (that was passed in)
		//   into a hashtable
		PackageAttributeAccessBean abPackageAttribute;
		String strValue;
		Hashtable hshResolveableAttributes = new Hashtable();
		int counter = 0;
			
		while (enAttributeList.hasMoreElements())
		{
			//Need to itterate through the list of distinct attributes
			//for each one, fish out the value from the passed in AttributeNVPs
			//if the attribute was not suplied, thow an error, else store the NVP in resolvableAttributes

			abPackageAttribute = (PackageAttributeAccessBean) enAttributeList.nextElement();
		    strValue = (String)getAttributeNVPs().get(abPackageAttribute.getAttributeIDInEJBType());

		    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
					"attribute is: " + abPackageAttribute.getAttributeID() + " (" + abPackageAttribute.getAttribute().getName() + ")");

			ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
					"trying to match value: " + strValue);

			if (strValue == null)
			{
				//the distinct attribute is missing
			    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
					"At least one distinct attribute is missing for Package " + getPackageId().toString() 
					+ ". Calling error handler: " + ErrorMissingAttributeCmd.ERRTASK_NAME 
					+ " with error code: " + ErrorMissingAttributeCmd._ERR_CODE_MISSING_ATTR);

			    TypedProperty temp = new TypedProperty();
				temp.put(com.ibm.commerce.server.ECConstants.EC_ERROR_CODE, ErrorMissingAttributeCmd._ERR_CODE_MISSING_ATTR);
				temp.put(com.ibm.commerce.server.ECConstants.EC_PRODUCT, getPackageId().toString());

				ECApplicationException exc =  new ECApplicationException(
													ECMessage._ERR_GETTING_SKU,
					                                this.getClass().getName(),"performExecute",
					                                ErrorMissingAttributeCmd.ERRTASK_NAME,
					                                temp
					                                );
				
				throw exc;								
			}
			else
			{
				//add the attribute/value pair to the hashtable
				counter++;
				hshResolveableAttributes.put(abPackageAttribute, strValue);
			}
				 
		}

		try
		{
			//Now that we have all the relevant atribute/value pairs, we can resolve the package item number
			setPackageItemId(abPackage.findByAttributes(hshResolveableAttributes).getCatalogEntryReferenceNumberInEJBType());
			
		} catch (Exception e) {
			
		    ECTrace.trace(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute", 
				"Package resolution failed for Package: " + getPackageId()  
				+ " Calling error handler: " + ErrorProductAttributeCmd.ERRTASK_NAME 
				+ " with error code: " + ErrorProductAttributeCmd._ERR_CODE_BAD_PROD_ATTR);

		    TypedProperty temp = new TypedProperty();
			temp.put(com.ibm.commerce.server.ECConstants.EC_ERROR_CODE, ErrorProductAttributeCmd._ERR_CODE_BAD_PROD_ATTR);
			temp.put(com.ibm.commerce.server.ECConstants.EC_PRODUCT, getPackageId().toString());

			ECApplicationException exc =  new ECApplicationException(
												ECMessage._ERR_GETTING_SKU,
				                                this.getClass().getName(),"performExecute",
				                                ErrorProductAttributeCmd.ERRTASK_NAME,
				                                temp
				                                );
			throw exc;								
		}
			
			
	} catch (ECException e) {
		throw e;
	} catch (Exception e) {
	    throw new ECSystemException(ECMessage._ERR_GENERIC,this.getClass().getName(),"performExecute",ECMessageHelper.generateMsgParms(e.getMessage()),e);
	}
		
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "performExecute");
}
public void reset() {
	super.reset();

	ihshAttributeNVPs = null;
	inPackageItemNumber = null;
	inUnresolvedPackageNumber = null;
}
/**
 * Sets the hashtable that represents the PackageAttribute names and values that will be used to perform the Package resolution. 
 * @param ahshAttrNVPs java.util.Hashtable
 */
public void setAttributeNVPs(Hashtable ahshAttrNVPs) 
{
	ihshAttributeNVPs = ahshAttrNVPs;	
}
/**
 * Sets the PackageId that represents the primary key of the Package that is to be resolved,
 * @param anPackageId java.lang.Long
 */
public void setPackageId (Long anPackageNumber)
{
	inUnresolvedPackageNumber = anPackageNumber;
}
/**
 * Sets the PackageId that represents the primary key of the resolved Package
 * @param anItemNumber java.lang.Long
 */
protected void setPackageItemId (Long anItemNumber)
{
	inPackageItemNumber = anItemNumber;
}
public void validateParameters() throws ECException 
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "validateParameters");
	
	super.validateParameters();

	if (getPackageId() == null) 
		throw new ECSystemException(ECMessage._ERR_DIDNT_SET_PRODUCT, this.getClass().getName(), "validateParameters");


	if (getAttributeNVPs() == null)
		setAttributeNVPs(new Hashtable());
		
	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_CATALOG, this.getClass().getName(), "validateParameters");
}

/**
 * No value constructor.
 */
public PackageResolveSkuCmdImpl() 
{
	super();
}
}
