//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////


import com.ibm.commerce.command.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.contract.objects.*;
import com.ibm.commerce.contract.objimpl.*;
import com.ibm.commerce.common.objects.*;
import com.ibm.commerce.common.objimpl.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.objimpl.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.price.commands.*;
import com.ibm.commerce.price.utils.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.catalog.objimpl.*;
import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.utils.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.registry.*;
import com.ibm.commerce.contract.util.*;
import com.ibm.util.*;
import java.math.*;
import java.rmi.*;
import java.sql.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.*;

/**
 * This <code>ResolvePriceListsCmd</code> Task Command Implementation 
 * retrieves the price lists referenced by each of the input price business
 * policies.
 * 
 * <p>
 * If the command is successful in retreiving the price lists, it will return 
 * a list of price lists with their assciated business policies. 
 *
 * <p>
 * If unsuccessful, it wil return an empty list.
 *
 * <p><strong>Behaviour</strong>
 * <p>Resolve the reference number of the price lists referenced by the
 * input list of price business policies.
 */
public class ResolvePriceListsCmdImpl extends com.ibm.commerce.command.BusinessPolicyCommandImpl implements ResolvePriceListsCmd {
	
	public static final String COPYRIGHT						= com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
				
	private static final String CLASS_NAME						= "com.ibm.commerce.price.commands.ResolvePriceListsCmdImpl";
	private static final String VALIDATE_PARAMETERS				= "validateParameters";
	private static final String PERFORM_EXECUTE					= "performExecute";
	private static final String RESOLVE_PRICELISTS				= "resolvePriceLists";
	
	private Hashtable iPolicyPriceListHash				= null;
	private BusinessPolicyAccessBean[] iPricePolicyABs	= null;
		
	public ResolvePriceListsCmdImpl() { super(); }
/**
 * Returns a list of price business policies and their associated price lists.
 */
public Hashtable getPolicyPriceLists() { return iPolicyPriceListHash; }
/**
 * Returns the access beans of the input price business policies.
 */
private BusinessPolicyAccessBean[] getPricePolicyABs() { return iPricePolicyABs; }
/**
 * Executes main business logic of the command.
 */
public void performExecute() throws ECException {
		
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, PERFORM_EXECUTE);
		
	//checkParameters();
	
	iPolicyPriceListHash = new Hashtable();
		
	if(iPricePolicyABs != null && iPricePolicyABs.length > 0)
		resolvePriceLists();
					
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, PERFORM_EXECUTE);

}
/**
 * Obtains the price list referenced by a list of input price business policies.
 */
protected void resolvePriceLists() throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_PRICELISTS);

	try {

		for(int i = 0; i<iPricePolicyABs.length; i++) {
			BusinessPolicyRegistryEntry policyRegEntry = 
				BusinessPolicyRegistry.singleton().getPolicyRegistryEntry(iPricePolicyABs[i].getPolicyId());
		
			TypedProperty param = policyRegEntry.getProperties();
			if(param != null && param.size() > 0) {
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), RESOLVE_PRICELISTS, 
					"policy id and properties: " + iPricePolicyABs[i].getPolicyId() +  "," + param.toString());
						
				String priceListName = param.getString(PriceCalculationConstants.PRICE_POLICY_OBJECT_NAME);	
				Long priceListOwner = param.getLong(PriceCalculationConstants.PRICE_POLICY_OWNER);

				if(priceListName != null && priceListName.length() != 0 && priceListOwner != null) {
					Long priceListId = (Long)com.ibm.commerce.server.WcsApp.priceListRegistry.getPriceListId(priceListOwner, priceListName);
					if(priceListId != null)
						iPolicyPriceListHash.put(iPricePolicyABs[i].getPolicyId(), priceListId);
				}
			}
		}

	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, RESOLVE_PRICELISTS, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, RESOLVE_PRICELISTS, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, RESOLVE_PRICELISTS, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, RESOLVE_PRICELISTS, ex);
	}
		
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_PRICELISTS);

}
/**
 * Sets the input price business policy access beans for determining the price lists.
 */
public void setPricePolicyABs(BusinessPolicyAccessBean[] aPricePolicyABs) {
	iPricePolicyABs = aPricePolicyABs;
}
/**
 * Sets the request properties for the policy.
 *
 * @param requestProperties the request properties for the policy
 */
public void setRequestProperties(com.ibm.commerce.datatype.TypedProperty requestProperties) {
	this.requestProperties = requestProperties;
}
/**
 * Checks mandatory parameters.
 */
public void validateParameters() throws ECException {
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, VALIDATE_PARAMETERS);

	if (getPricePolicyABs() == null) {
		// PriceList?: add PricePolicy constant
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME, VALIDATE_PARAMETERS , new Object[] {PriceCalculationConstants.PRICELISTS});
	}
	
			
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, VALIDATE_PARAMETERS);
	
}
}
